#include "ssd1306_driver.h"


//######################################## Register
#define START_SSD1306_REGISTER {

	#define SSD1306_DISPLAYALLON_RESUME					0xA4
	#define SSD1306_DISPLAYALLON						0xA5
	#define SSD1306_NORMALDISPLAY						0xA6
	#define SSD1306_INVERTDISPLAY						0xA7
	#define SSD1306_DISPLAYOFF							0xAE
	#define SSD1306_DISPLAYON							0xAF

	#define SSD1306_SETDISPLAYOFFSET					0xD3
	#define SSD1306_SETCOMPINS							0xDA
	#define SSD1306_SETVCOMDETECT						0xDB
	#define SSD1306_SETDISPLAYCLOCKDIV					0xD5
	#define SSD1306_SETPRECHARGE						0xD9
	#define SSD1306_SETMULTIPLEX						0xA8
	#define SSD1306_SETLOWCOLUMN						0x00
	#define SSD1306_SETHIGHCOLUMN						0x10
	#define SSD1306_SETSTARTLINE						0x40
	#define SSD1306_SETCONTRAST							0x81

	#define SSD1306_MEMORYMODE							0x20
	#define SSD1306_COLUMNADDR							0x21
	#define SSD1306_PAGEADDR							0x22
	#define SSD1306_COMSCANINC							0xC0
	#define SSD1306_COMSCANDEC							0xC8
	#define SSD1306_SEGREMAP							0xA0
	#define SSD1306_CHARGEPUMP							0x8D
	#define SSD1306_EXTERNALVCC							0x1
	#define SSD1306_SWITCHCAPVCC						0x2

	// Scrolling #defines
	#define SSD1306_ACTIVATE_SCROLL						0x2F
	#define SSD1306_DEACTIVATE_SCROLL					0x2E
	#define SSD1306_SET_VERTICAL_SCROLL_AREA			0xA3
	#define SSD1306_RIGHT_HORIZONTAL_SCROLL				0x26
	#define SSD1306_LEFT_HORIZONTAL_SCROLL				0x27
	#define SSD1306_VERTICAL_AND_RIGHT_HORIZONTAL_SCROLL 0x29
	#define SSD1306_VERTICAL_AND_LEFT_HORIZONTAL_SCROLL 0x2A

#define END_SSD1306_REGISTER }


#define bitRead(value, bit) (((value) >> (bit)) & 0x01)

void ssd1306_write_cmd(ssd1306_data_t *ssd_data, uint8_t cmd);

//##########################################################################################
void ssd1306_init(ssd1306_data_t *ssd_data, TWI_Master_t *twi, uint8_t address)
{
	ssd_data->addr		= address;
	ssd_data->twi		= twi;
// 	ssd_data->width		= 128;	
// 	ssd_data->height	= 64;		
//	ssd_data->font		= font;
	ssd_data->color		= WHITE;		
	ssd_data->alignment = TEXT_ALIGN_LEFT;	
	
	
	// Init sequence for 128x64 OLED module
	ssd1306_write_cmd(ssd_data,  SSD1306_DISPLAYOFF); 			// 0xAE
	ssd1306_write_cmd(ssd_data,  SSD1306_SETDISPLAYCLOCKDIV); 	// 0xD5
	ssd1306_write_cmd(ssd_data,  0xF0); 						// 0xF0
	ssd1306_write_cmd(ssd_data,  SSD1306_SETMULTIPLEX); 		// 0xA8
	ssd1306_write_cmd(ssd_data,  0x3F);							// 0x3F
	ssd1306_write_cmd(ssd_data,  SSD1306_SETDISPLAYOFFSET); 	// 0xD3
	ssd1306_write_cmd(ssd_data,  0x00); 						// 0x00	
	ssd1306_write_cmd(ssd_data,  SSD1306_SETSTARTLINE); 		// 0x40	
	ssd1306_write_cmd(ssd_data,  SSD1306_CHARGEPUMP); 			// 0x8D
	ssd1306_write_cmd(ssd_data,  0x14);							// 0x14
	ssd1306_write_cmd(ssd_data,  SSD1306_MEMORYMODE); 			// 0x20 
	ssd1306_write_cmd(ssd_data,  0x00); 						// 0x00	
	ssd1306_write_cmd(ssd_data,  SSD1306_SEGREMAP); 			// 0xA0
	ssd1306_write_cmd(ssd_data,  SSD1306_COMSCANINC);			// 0xC0
	ssd1306_write_cmd(ssd_data,  SSD1306_SETCOMPINS); 			// 0xDA
	ssd1306_write_cmd(ssd_data,  0x12);	
	ssd1306_write_cmd(ssd_data,  SSD1306_SETCONTRAST); 			// 0x81
	ssd1306_write_cmd(ssd_data,  0xCF);
	ssd1306_write_cmd(ssd_data,  SSD1306_SETPRECHARGE); 		// 0xd9
	ssd1306_write_cmd(ssd_data,  0xF1);	
	ssd1306_write_cmd(ssd_data,  SSD1306_DISPLAYALLON_RESUME); 	// 0xA4	 
	ssd1306_write_cmd(ssd_data,  SSD1306_NORMALDISPLAY); 		// 0xA6	
	ssd1306_write_cmd(ssd_data,  SSD1306_DEACTIVATE_SCROLL); 	// 0x2E
	ssd1306_write_cmd(ssd_data,  SSD1306_DISPLAYON);			// 0xAF	
	
	ssd1306_resetDisplay(ssd_data);
}

void ssd1306_write_cmd(ssd1306_data_t *ssd_data, uint8_t cmd)
{
  	uint8_t sendBuffer[] = {0x80, cmd};
 	TWI_MasterWriteRead( ssd_data->twi, ssd_data->addr, &sendBuffer[0] , 2, 0);	// Daten des slaves anfordern
}

void ssd1306_resetDisplay(ssd1306_data_t *ssd_data) 
{
	ssd1306_clear(ssd_data);
	ssd1306_update(ssd_data);
}

void ssd1306_displayOn(ssd1306_data_t *ssd_data) {
	ssd1306_write_cmd(ssd_data, 0xaf);        //display on
}

void ssd1306_displayOff(ssd1306_data_t *ssd_data) {
	ssd1306_write_cmd(ssd_data, 0xae);          //display off
}

void ssd1306_displayEn(ssd1306_data_t *ssd_data, uint8_t enable)
{
	if (enable)
	{	ssd1306_write_cmd(ssd_data, 0xaf);        //display on

	} 
	else
	{	ssd1306_write_cmd(ssd_data, 0xae);          //display off

	}
}

void ssd1306_setContrast(ssd1306_data_t *ssd_data, uint8_t contrast) {
	ssd1306_write_cmd(ssd_data, SSD1306_SETCONTRAST);
	ssd1306_write_cmd(ssd_data, contrast);
}

void ssd1306_setInvert(ssd1306_data_t *ssd_data, uint8_t state)
{
	if(state)	ssd1306_write_cmd(ssd_data, SSD1306_INVERTDISPLAY);
	else		ssd1306_write_cmd(ssd_data, SSD1306_NORMALDISPLAY);
}

void ssd1306_flipScreenVertically(ssd1306_data_t *ssd_data) {
	ssd1306_write_cmd(ssd_data, 0xA0 | 0x1);      //SEGREMAP   //Rotate screen 180 deg
	ssd1306_write_cmd(ssd_data, 0xC8);            //COMSCANDEC  Rotate screen 180 Deg
}

void ssd1306_clear(ssd1306_data_t *ssd_data) 
{
	memset(&ssd_data->buffer, BLACK, DISPLAY_BUFFER_SIZE);
}

void ssd1306_update(ssd1306_data_t *ssd_data) 
{	
	ssd1306_write_cmd(ssd_data, SSD1306_COLUMNADDR);		// 0x21
	ssd1306_write_cmd(ssd_data, 0x00);						// 0x00 Column start address (0 = reset)
	ssd1306_write_cmd(ssd_data, 0x7F);						// 0x7F ssd_data->width-1);		// Column end address (127 = reset)

	ssd1306_write_cmd(ssd_data, SSD1306_PAGEADDR);			// 0x22
	ssd1306_write_cmd(ssd_data, 0x00);						// 0x00 Page start address (0 = reset)
	ssd1306_write_cmd(ssd_data, 0x07);						// 0x07 ssd_data->height/8-1);		// Page end address

	while (ssd_data->twi->status != TWIM_STATUS_READY)
	{}
			 
	ssd_data->twi->writeData[0]=0x40;
	memcpy(&ssd_data->twi->writeData[1], &ssd_data->buffer, 1024) ;
	
	TWI_MasterWriteAll(ssd_data->twi, ssd_data->addr);
}

//##########################################################################################


// Header Values
#define JUMPTABLE_BYTES 4

#define JUMPTABLE_LSB   1
#define JUMPTABLE_SIZE  2
#define JUMPTABLE_WIDTH 3
#define JUMPTABLE_START 4

#define WIDTH_POS 0
#define HEIGHT_POS 1
#define FIRST_CHAR_POS 2
#define CHAR_NUM_POS 3



void ssd1306_drawString(ssd1306_data_t *ssd_data, int16_t x, int16_t y, char* text) 
{
	//text = utf8AsciiStr(text);
	unsigned char currentByte;
	int charX, charY;
	int currentBitCount;
	int charCode;
	int currentCharWidth;
	int currentCharStartPos;
	int cursorX = 0;
	int numberOfChars = pgm_read_byte(ssd_data->font + CHAR_NUM_POS);
	// iterate over string
	int firstChar = pgm_read_byte(ssd_data->font + FIRST_CHAR_POS);
	int charHeight = pgm_read_byte(ssd_data->font + HEIGHT_POS);
	int currentCharByteNum = 0;
	int startX = 0;
	int startY = y;

	if (ssd_data->alignment == TEXT_ALIGN_LEFT) 
	{
		startX = x;
	} 
	else if (ssd_data->alignment == TEXT_ALIGN_CENTER) 
	{
		int width = ssd1306_getStringWidth(ssd_data, text);
		startX = x - width / 2;
	} 
	else if (ssd_data->alignment == TEXT_ALIGN_RIGHT) 
	{
		int width = ssd1306_getStringWidth(ssd_data, text);
		startX = x - width;
	}

	for (int j=0; j < strlen(text); j++) 
	{

		charCode = text[j]-0x20;

		currentCharWidth = pgm_read_byte(ssd_data->font + CHAR_WIDTH_START_POS + charCode);
		// Jump to font data beginning
		currentCharStartPos = CHAR_WIDTH_START_POS + numberOfChars;

		for (int m = 0; m < charCode; m++) {

			currentCharStartPos += pgm_read_byte(ssd_data->font + CHAR_WIDTH_START_POS + m)  * charHeight / 8 + 1;
		}

		currentCharByteNum = ((charHeight * currentCharWidth) / 8) + 1;
		// iterate over all bytes of character
		for (int i = 0; i < currentCharByteNum; i++) {

			currentByte = pgm_read_byte(ssd_data->font + currentCharStartPos + i);
			//Serial.println(String(charCode) + ", " + String(currentCharWidth) + ", " + String(currentByte));
			// iterate over all bytes of character
			for(int bit = 0; bit < 8; bit++) {
				//int currentBit = bitRead(currentByte, bit);

				currentBitCount = i * 8 + bit;

				charX = currentBitCount % currentCharWidth;
				charY = currentBitCount / currentCharWidth;

				if (bitRead(currentByte, bit)) {
					ssd1306_drawPixel(ssd_data, startX + cursorX + charX, startY + charY);
				}

			}
			//yield();
		}
		cursorX += currentCharWidth;
	}
}

void ssd1306_drawStringMaxWidth(ssd1306_data_t *ssd_data, int16_t x, int16_t y, uint16_t maxLineWidth,  char* text)			//strUser) 
{
	int currentLineWidth = 0;
	int startsAt = 0;
	int endsAt = 0;
	int lineNumber = 0;
	char currentChar = ' ';
	int lineHeight = pgm_read_byte(ssd_data->font + HEIGHT_POS);
	char* currentLine = "";
	
	for (int i = 0; i < strlen(text); i++) 
	{
		currentChar = text[i];
		if (currentChar == ' ' || currentChar == '-')
		{
			//char* lineCandidate; = text.substring(startsAt, i);
			char* lineCandidate;
			strncpy (&lineCandidate,text[startsAt], i-startsAt );
			if (ssd1306_getStringWidth(ssd_data, lineCandidate) <= maxLineWidth)
			{
				endsAt = i;
			}
			else
			{
				char* tmp;
				strncpy(&tmp,text[startsAt], endsAt-endsAt );
				ssd1306_drawString(ssd_data, x, y + lineNumber * lineHeight, tmp);
				lineNumber++;
				startsAt = endsAt + 1;
			}
		}
	}
	char* tmp;
	strncpy(&tmp,text[startsAt], strlen(text) );
	ssd1306_drawString(ssd_data, x, y + lineNumber * lineHeight, tmp);
}


uint16_t ssd1306_getStringWidthLen(ssd1306_data_t *ssd_data, char* text, uint16_t length) 
{
	uint16_t firstChar        = pgm_read_byte(ssd_data->font + FIRST_CHAR_POS);

	uint16_t stringWidth = 0;
	uint16_t maxWidth = 0;

	while (length--) {
		stringWidth += pgm_read_byte(ssd_data->font + JUMPTABLE_START + (text[length] - firstChar) * JUMPTABLE_BYTES + JUMPTABLE_WIDTH);
		if (text[length] == 10) {
			maxWidth = max(maxWidth, stringWidth);
			stringWidth = 0;
		}
	}

	return max(maxWidth, stringWidth);
}
uint16_t ssd1306_getStringWidth(ssd1306_data_t *ssd_data, char* text)			//strUser) 
{
  //text = utf8AsciiStr(text);
  int stringWidth = 0;
  char charCode;
  for (int j=0; j < strlen(text); j++) 
  {
	  charCode = text[j]-0x20;
	  stringWidth += pgm_read_byte(ssd_data->font + CHAR_WIDTH_START_POS + charCode);
  }
  return stringWidth;
}



char utf8AsciiChar(char ascii) 
{
	static char lastChar;

	if ( ascii < 128 ) { // Standard ASCII-set 0..0x7F handling
		lastChar = 0;
		return ascii;
	}

	char last = lastChar;   // get last char
	lastChar = ascii;

	switch (last) 
	{    // conversion depnding on first UTF8-character
		case 0xC2: return  (ascii);  break;
		case 0xC3: return  (ascii | 0xC0);  break;
		case 0x82: if (ascii == 0xAC) return (0x80);    // special case Euro-symbol
	}

	return  0; // otherwise: return zero, if character has to be ignored
}

char* utf8AsciiStr(char *str) 
{
    char * r= "";    
    for (int i=0; i<strlen(str); i++)
    {
	    char c = utf8AsciiChar(str[i]);
	    if (c!=0) r+=c;
    }
    return r;
}

//##########################################################################################

void ssd1306_setTextAlignment(ssd1306_data_t *ssd_data, uint8_t textAlignment) 
{
	ssd_data->alignment = textAlignment;
}

void ssd1306_setFont(ssd1306_data_t *ssd_data, const char *fontData) 
{
	ssd_data->font = fontData;
}

void ssd1306_setColor(ssd1306_data_t *ssd_data, uint8_t color)
{
	ssd_data->color = color;
}

//##########################################################################################
void ssd1306_drawPixel(ssd1306_data_t *ssd_data, int16_t x, int16_t y)
{
	if (x >= 0 && x < 128 && y >= 0 && y < 64)
	{
		switch (ssd_data->color)
		{
			case WHITE:		ssd_data->buffer[x + (y / 8) * DISPLAY_WIDTH] |=  (1 << (y & 7)); break;
			case BLACK:		ssd_data->buffer[x + (y / 8) * DISPLAY_WIDTH] &= ~(1 << (y & 7)); break;
			case INVERSE:	ssd_data->buffer[x + (y / 8) * DISPLAY_WIDTH] ^=  (1 << (y & 7)); break;
		}
	}
}
void ssd1306_drawLine(ssd1306_data_t *ssd_data, int16_t x0, int16_t y0, int16_t x1, int16_t y1) 
{
	int16_t steep = abs(y1 - y0) > abs(x1 - x0);
	if (steep) {
		_swap_int16_t(x0, y0);
		_swap_int16_t(x1, y1);
	}

	if (x0 > x1) {
		_swap_int16_t(x0, x1);
		_swap_int16_t(y0, y1);
	}

	int16_t dx, dy;
	dx = x1 - x0;
	dy = abs(y1 - y0);

	int16_t err = dx / 2;
	int16_t ystep;

	if (y0 < y1) {
		ystep = 1;
		} else {
		ystep = -1;
	}

	for (; x0<=x1; x0++) {
		if (steep) {
			ssd1306_drawPixel(ssd_data, y0, x0);
			} else {
			ssd1306_drawPixel(ssd_data, x0, y0);
		}
		err -= dy;
		if (err < 0) {
			y0 += ystep;
			err += dx;
		}
	}
}
void ssd1306_drawHorizontalLine(ssd1306_data_t *ssd_data, int16_t x, int16_t y, int16_t length) {
	if (y < 0 || y >= DISPLAY_HEIGHT) { return; }

	if (x < 0) {
		length += x;
		x = 0;
	}

	if ( (x + length) > DISPLAY_WIDTH) {
		length = (DISPLAY_WIDTH - x);
	}

	if (length <= 0) { return; }

	uint8_t * bufferPtr = ssd_data->buffer;
	bufferPtr += (y >> 3) * DISPLAY_WIDTH;
	bufferPtr += x;

	uint8_t drawBit = 1 << (y & 7);

	switch (ssd_data->color) {
		case WHITE:   while (length--) {
			*bufferPtr++ |= drawBit;
		}; break;
		case BLACK:   drawBit = ~drawBit;   while (length--) {
			*bufferPtr++ &= drawBit;
		}; break;
		case INVERSE: while (length--) {
			*bufferPtr++ ^= drawBit;
		}; break;
	}
}
void ssd1306_drawVerticalLine(ssd1306_data_t *ssd_data, int16_t x, int16_t y, int16_t length) {
	if (x < 0 || x > DISPLAY_WIDTH) return;

	if (y < 0) {
		length += y;
		y = 0;
	}

	if ( (y + length) > DISPLAY_HEIGHT) {
		length = (DISPLAY_HEIGHT - y);
	}

	if (length <= 0) return;


	uint8_t yOffset = y & 7;
	uint8_t drawBit;
	uint8_t *bufferPtr = ssd_data->buffer;

	bufferPtr += (y >> 3) * DISPLAY_WIDTH;
	bufferPtr += x;

	if (yOffset) {
		yOffset = 8 - yOffset;
		drawBit = ~(0xFF >> (yOffset));

		if (length < yOffset) {
			drawBit &= (0xFF >> (yOffset - length));
		}

		switch (ssd_data->color) {
			case WHITE:   *bufferPtr |= drawBit; break;
			case BLACK:   *bufferPtr &= drawBit; break;
			case INVERSE: *bufferPtr ^= drawBit; break;
		}

		if (length < yOffset) return;

		length -= yOffset;
		bufferPtr += DISPLAY_WIDTH;
	}

	if (length >= 8) {
		switch (ssd_data->color) {
			case WHITE:
			case BLACK:
			drawBit = (ssd_data->color == WHITE) ? 0xFF : 0x00;
			do {
				*bufferPtr = drawBit;
				bufferPtr += DISPLAY_WIDTH;
				length -= 8;
			} while (length >= 8);
			break;
			case INVERSE:
			do {
				*bufferPtr = ~(*bufferPtr);
				bufferPtr += DISPLAY_WIDTH;
				length -= 8;
			} while (length >= 8);
			break;
		}
	}

	if (length > 0) {
		drawBit = (1 << length & 7) - 1;
		switch (ssd_data->color) {
			case WHITE:   *bufferPtr |= drawBit; break;
			case BLACK:   *bufferPtr &= drawBit; break;
			case INVERSE: *bufferPtr ^= drawBit; break;
		}
	}
}
void ssd1306_drawRect(ssd1306_data_t *ssd_data, int16_t x, int16_t y, int16_t width, int16_t height)
{
  ssd1306_drawHorizontalLine(ssd_data, x, y, width);
  ssd1306_drawVerticalLine(ssd_data, x, y, height);
  ssd1306_drawVerticalLine(ssd_data, x + width, y, height);
  ssd1306_drawHorizontalLine(ssd_data, x, y + height, width);
}
void ssd1306_fillRect(ssd1306_data_t *ssd_data, int16_t x, int16_t y, int16_t width, int16_t height) 
{
	for (int i = x; i < x + width; i++) {
		for (int j = y; j < y + height; j++) {
			ssd1306_drawPixel(ssd_data, i, j);
		}
	}
}


void ssd1306_drawCircle(ssd1306_data_t *ssd_data, int16_t x0, int16_t y0, int16_t radius) 
{
	int16_t x = 0, y = radius;
	int16_t dp = 1 - radius;
	do {
		if (dp < 0)
		dp = dp + 2 * (++x) + 3;
		else
		dp = dp + 2 * (++x) - 2 * (--y) + 5;

		ssd1306_drawPixel(ssd_data, x0 + x, y0 + y);     //For the 8 octants
		ssd1306_drawPixel(ssd_data, x0 - x, y0 + y);
		ssd1306_drawPixel(ssd_data, x0 + x, y0 - y);
		ssd1306_drawPixel(ssd_data, x0 - x, y0 - y);
		ssd1306_drawPixel(ssd_data, x0 + y, y0 + x);
		ssd1306_drawPixel(ssd_data, x0 - y, y0 + x);
		ssd1306_drawPixel(ssd_data, x0 + y, y0 - x);
		ssd1306_drawPixel(ssd_data, x0 - y, y0 - x);

	} while (x < y);

	ssd1306_drawPixel(ssd_data, x0 + radius, y0);
	ssd1306_drawPixel(ssd_data, x0, y0 + radius);
	ssd1306_drawPixel(ssd_data, x0 - radius, y0);
	ssd1306_drawPixel(ssd_data, x0, y0 - radius);
}
void ssd1306_fillCircle(ssd1306_data_t *ssd_data, int16_t x0, int16_t y0, int16_t radius) 
{
	int16_t x = 0, y = radius;
	int16_t dp = 1 - radius;
	do {
		if (dp < 0)
		dp = dp + 2 * (++x) + 3;
		else
		dp = dp + 2 * (++x) - 2 * (--y) + 5;

		ssd1306_drawHorizontalLine(ssd_data, x0 - x, y0 - y, 2*x);
		ssd1306_drawHorizontalLine(ssd_data, x0 - x, y0 + y, 2*x);
		ssd1306_drawHorizontalLine(ssd_data, x0 - y, y0 - x, 2*y);
		ssd1306_drawHorizontalLine(ssd_data, x0 - y, y0 + x, 2*y);


	} while (x < y);
	ssd1306_drawHorizontalLine(ssd_data, x0 - radius, y0, 2 * radius);

}
void ssd1306_drawProgressBar(ssd1306_data_t *ssd_data, uint16_t x, uint16_t y, uint16_t width, uint16_t height, uint8_t progress) 
{
	uint16_t radius = height / 2;
	uint16_t innerRadius = radius - 3;
	ssd1306_setColor(ssd_data, WHITE);
	ssd1306_drawCircle(ssd_data, x + radius, y + radius, radius);
	ssd1306_drawRect(ssd_data, x+radius, y, width - 2*radius, height);
	ssd1306_drawCircle(ssd_data, x + width - radius, y + radius, radius);
	ssd1306_setColor(ssd_data, BLACK);
	ssd1306_fillRect(ssd_data, x+radius, y+1, width - 2*radius + 1, height - 1);
	ssd1306_setColor(ssd_data, WHITE);
	uint16_t maxProgressWidth = (width - 2 * radius) * progress / 100;
	for (uint16_t i = 0; i < maxProgressWidth; i++) 
	{
		ssd1306_fillCircle(ssd_data, x + radius + i, y + radius, innerRadius);
	}

}
void ssd1306_drawFastImage(ssd1306_data_t *ssd_data, int16_t xMove, int16_t yMove, int16_t width, int16_t height, const uint8_t *image) 
{
	//drawInternal(xMove, yMove, width, height, image, 0, 0);
}
void ssd1306_drawXbm(ssd1306_data_t *ssd_data, int16_t xMove, int16_t yMove, int16_t width, int16_t height, const uint8_t *xbm)
{
	int16_t widthInXbm = (width + 7) / 8;
	uint8_t data =0;

	for(int16_t y = 0; y < height; y++) 
	{
		for(int16_t x = 0; x < width; x++ ) 
		{
			if (x & 7) {
				data >>= 1; // Move a bit
				} else {  // Read new data every 8 bit
				data = pgm_read_byte(xbm + (x / 8) + y * widthInXbm);
			}
			// if there is a bit draw it
			if (data & 0x01) {
				ssd1306_drawPixel(ssd_data, xMove + x, yMove + y);
			}
		}
	}
}
void ssd1306_drawBitmap(ssd1306_data_t *ssd_data,int16_t x, int16_t y, int16_t width, int16_t height, const uint8_t *bitmap) 
{
	for (int i = 0; i < width * height / 8; i++ ){
		unsigned char charColumn = 255 - pgm_read_byte(bitmap + i);
		for (int j = 0; j < 8; j++) {
			int targetX = i % width + x;
			int targetY = (i / (width)) * 8 + j + y;
			if (bitRead(charColumn, j)) {
				ssd1306_drawPixel(ssd_data, targetX, targetY);
			}
		}
	}
}