/*
 * ssd1306_driver.h
 *
 * Created: 14.05.2014 
 * Author: Martin Junghans
 */ 


#ifndef SSD1306_DRIVER_H_
#define SSD1306_DRIVER_H_

#include <stdlib.h>
#include <avr/io.h>
#include <stdio.h>
#include <stddef.h>
#include <string.h>
#include <avr/interrupt.h>


#include "twi_master_driver.h"
#include "macros.h"



#ifndef _swap_int16_t
#define _swap_int16_t(a, b) { int16_t t = a; a = b; b = t; }
#endif

#define BLACK		0
#define WHITE		1
#define INVERSE		2

#define TEXT_ALIGN_LEFT			0
#define TEXT_ALIGN_CENTER		1
#define TEXT_ALIGN_RIGHT		2
#define TEXT_ALIGN_CENTER_BOTH  3
	
#define WIDTH_POS				0
#define HEIGHT_POS				1
#define FIRST_CHAR_POS			2
#define CHAR_NUM_POS			3
#define CHAR_WIDTH_START_POS	4
	
#define DISPLAY_WIDTH			128
#define DISPLAY_HEIGHT			64
#define DISPLAY_BUFFER_SIZE		(DISPLAY_WIDTH * DISPLAY_HEIGHT / 8)

typedef struct ssd1306_data_t	{
	uint8_t			addr;
	TWI_Master_t	*twi;
	uint8_t			buffer[DISPLAY_BUFFER_SIZE];
	uint8_t			alignment;
	uint8_t			color;
	const char	*font;
}ssd1306_data_t;


		
void ssd1306_init(ssd1306_data_t *ssd_data, TWI_Master_t *twi, uint8_t address);

// Display functions 
void ssd1306_resetDisplay(ssd1306_data_t *ssd_data);
void ssd1306_displayEn(ssd1306_data_t *ssd_data, uint8_t enable);
void ssd1306_displayOn(ssd1306_data_t *ssd_data);
void ssd1306_displayOff(ssd1306_data_t *ssd_data);
void ssd1306_clear(ssd1306_data_t *ssd_data);			// Clear the local pixel buffer
void ssd1306_setContrast(ssd1306_data_t *ssd_data, uint8_t contrast);	// Set display contrast
void ssd1306_flipScreenVertically(ssd1306_data_t *ssd_data);			// Turn the display upside down
void ssd1306_setInvert(ssd1306_data_t *ssd_data, uint8_t state);		// Inverted/Normal display mode
void ssd1306_update(ssd1306_data_t *ssd_data);							// Write the buffer to the display memory


// Drawing functions
void ssd1306_setColor(ssd1306_data_t *ssd_data, uint8_t color);    // Sets the color of all pixel operations
void ssd1306_drawPixel(ssd1306_data_t *ssd_data, int16_t x, int16_t y);// Draw a pixel at given position
void ssd1306_drawLine(ssd1306_data_t *ssd_data, int16_t x0, int16_t y0, int16_t x1, int16_t y1);// Draw a line from position 0 to position 1
void ssd1306_drawHorizontalLine(ssd1306_data_t *ssd_data, int16_t x, int16_t y, int16_t length);// Draw a line horizontally
void ssd1306_drawVerticalLine(ssd1306_data_t *ssd_data, int16_t x, int16_t y, int16_t length);// Draw a lin vertically
void ssd1306_drawRect(ssd1306_data_t *ssd_data, int16_t x, int16_t y, int16_t width, int16_t height);// Draw the border of a rectangle at the given location
void ssd1306_fillRect(ssd1306_data_t *ssd_data, int16_t x, int16_t y, int16_t width, int16_t height);// Fill the rectangle
void ssd1306_drawCircle(ssd1306_data_t *ssd_data, int16_t x, int16_t y, int16_t radius);// Draw the border of a circle
void ssd1306_fillCircle(ssd1306_data_t *ssd_data, int16_t x, int16_t y, int16_t radius);// Fill circle

void ssd1306_drawProgressBar(ssd1306_data_t *ssd_data, uint16_t x, uint16_t y, uint16_t width, uint16_t height, uint8_t progress);// Draws a rounded progress bar with the outer dimensions given by width and height. Progress is a unsigned byte value between 0 and 100
void ssd1306_drawFastImage(ssd1306_data_t *ssd_data, int16_t x, int16_t y, int16_t width, int16_t height, const uint8_t *image);// Draw a bitmap in the internal image format
void ssd1306_drawXbm(ssd1306_data_t *ssd_data, int16_t x, int16_t y, int16_t width, int16_t height, const uint8_t *xbm);// Draw a XBM


void ssd1306_setFont(ssd1306_data_t *ssd_data, const char *fontData);
void ssd1306_setColor(ssd1306_data_t *ssd_data, uint8_t color);
void ssd1306_setTextAlignment(ssd1306_data_t *ssd_data, uint8_t textAlignment);


// Text functions 
void ssd1306_drawString(ssd1306_data_t *ssd_data, int16_t x, int16_t y, char *text);		// Draws a string at the given location

// Draws a String with a maximum width at the given location.
// If the given String is wider than the specified width
// The text will be wrapped to the next line at a space or dash
void ssd1306_drawStringMaxWidth(ssd1306_data_t *ssd_data, int16_t x, int16_t y, uint16_t maxLineWidth, char *text);

// Returns the width of the const char* with the current
// font settings
uint16_t ssd1306_getStringWidthLen(ssd1306_data_t *ssd_data, char* text, uint16_t length);

// Convencience method for the const char version
uint16_t ssd1306_getStringWidth(ssd1306_data_t *ssd_data, char* text);

// Specifies relative to which anchor point
// the text is rendered. Available constants:
// TEXT_ALIGN_LEFT, TEXT_ALIGN_CENTER, TEXT_ALIGN_RIGHT, TEXT_ALIGN_CENTER_BOTH
void ssd1306_setTextAlignment(ssd1306_data_t *ssd_data, uint8_t textAlignment);

// converts utf8 characters to extended ascii
// taken from http://playground.arduino.cc/Main/Utf8ascii
char utf8AsciiChar(char ascii);

// converts utf8 string to extended ascii
// taken from http://playground.arduino.cc/Main/Utf8ascii
char * utf8AsciiStr(char *s);

void drawStringInternal(ssd1306_data_t *ssd_data, int16_t xMove, int16_t yMove, char* text, uint16_t textLength, uint16_t textWidth);


#endif /* SSD1306_DRIVER_H_ */