
#include "main.h"

gui_ptr_t	gui_arr[] = {&gui_screen0};
uint32_t	timerGui=0, timerDac=0;
uint8_t		buffer[32];
	
#define ISR_START	{

ISR(USARTC0_RXC_vect)	{ USART_RXComplete(  &usartC0);}		
ISR(USARTC0_DRE_vect)	{ USART_DataRegEmpty(&usartC0);}
		
ISR(USARTD0_RXC_vect)	{ USART_RXComplete(  &usartD0);}
ISR(USARTD0_DRE_vect)	{ USART_DataRegEmpty(&usartD0);}
	
ISR(TWIE_TWIM_vect)		{ TWI_MasterInterruptHandler(&twiMasterE);	}
ISR(TCD0_OVF_vect)		{ rtc_isr();								}			// Timer 1000Hz	

#define ISR_END		}

void sys_clock_32MHz_ext(void)
{

	CLKSYS_XOSC_Config( OSC_FRQRANGE_12TO16_gc, false, OSC_XOSCSEL_XTAL_16KCLK_gc );// Quarz config	CLOCK_SRC_XOSC
	CLKSYS_Enable( OSC_XOSCEN_bm );													// enable oscillator
	
	CLKSYS_PLL_Config( OSC_PLLSRC_XOSC_gc, 8 );										// config PLL - Quarz + 8x -->128Mhz
	CLKSYS_Enable( OSC_PLLEN_bm );													// PLL enable
	CLKSYS_Prescalers_Config( CLK_PSADIV_1_gc, CLK_PSBCDIV_2_2_gc );				// Prescaller A, B, C --> Mainclock 32Mhz
	while ( !(CLKSYS_IsReady( OSC_PLLRDY_bm )) ) ;									// Wait for it to stabilize

	CLKSYS_Main_ClockSource_Select( CLK_SCLKSEL_PLL_gc );							// Source for Main Clock
	//CLKSYS_Disable( OSC_RC32MEN_bm| OSC_RC32KEN_bm | OSC_RC2MEN_bm );				// Turn off all the other clocks.
}

inline void sys_init(void)
{
	cli();	
	
	//####################################### Ports
	// High/Low
	PORTA.OUT=0b00000000;
	PORTB.OUT=0b00000010;			// analog	B0 - B3
	PORTC.OUT=0b00000000;			// digital	C4 - C7
	PORTD.OUT=0b00000000;			// 2 RX, 3 TX, 4 RES, 5 PWR, 6+7 USB
	PORTE.OUT=0b00000000;			// 2 RX, 3 TX, 4 RES, 5 PWR, 6+7 USB
	
	// Direction
	PORTA.DIR=0b00000000;
	PORTB.DIR=0b00000010;			// analog	B0 - B3
	PORTC.DIR=0b11110000;			// digital	C4 - C7
	PORTD.DIR=0b00110000;			// 2 RX, 3 TX, 4 RES, 5 PWR, 6+7 USB
	PORTE.DIR=0b00001100;			// 0 SDA, 1 SCL, 2 LIP, 3 SLEEP

	//####################################### Clock
	sys_clock_32MHz_ext();
	
	//####################################### Timer C1 --> 1000 Hz
	rtc_init(&TCD0);	// Timer C1 --> 1000 Hz

	//####################################### UART D0 --> Serial Communication
	cbi(PORTD.DIR,2);													// PD2 (RXD0) as input
	sbi(PORTD.DIR,3);													// PD3 (TXD0) as output
	usart_init(&usartD0, &USARTD0, USART_BAUD_SELECT_DOUBLE_SPEED(BAUDRATE_D0, F_CPU) , USART_DREINTLVL_LO_gc);	// 57600, 115200 128000 256000

	//####################################### TWI master
	sbi(PORTE.DIR,0);	cbi(PORTE.OUT,0);				// Ausgang	SDA
	sbi(PORTE.DIR,1);	cbi(PORTE.OUT,1);				// Ausgang	SCL
	PORTE.PIN0CTRL	|= PORT_OPC_PULLUP_gc;				// PullUp
	PORTE.PIN1CTRL	|= PORT_OPC_PULLUP_gc;				// PullUp
	TWI_MasterInit(&twiMasterE, &TWIE, TWI_MASTER_INTLVL_LO_gc, TWI_BAUD(F_CPU, 1000000));
	
	//####################################### SPI Master
	sbi(PORTC.DIR,4);	sbi(PORTC.OUT,4);				// output CS high (No slave addressed)
	SPI_MasterInit(&spiMasterC, &SPIC, &PORTC, false, SPI_MODE_0_gc, PMIC_MEDLVLEN_bm, false, SPI_PRESCALER_DIV4_gc);		// sensoren, ethernet, thermo ...
			
	//#######################################
	PMIC.CTRL = PMIC_LOLVLEN_bm | PMIC_MEDLVLEN_bm | PMIC_HILVLEN_bm;
	sei();	
}

int main(void)
{
	
	sys_init();
	
	ad5660_init (&ad5660_1, &spiMasterC, &PORTC, 2);
	ad5660_init (&ad5660_2, &spiMasterC, &PORTC, 3);	
	
	ssd1306_init(&ssd1306 , &twiMasterE, OLED_ADR); // OLED
	ssd1306_flipScreenVertically(&ssd1306);
	gui_init(&gui,&ssd1306, gui_arr, ARRAY_SIZE(gui_arr), 500,500);
	
	for(uint8_t i=0; i<100; i++)
	{
		gui_view(&gui, &gui_screenBoot);
		_delay_ms(10);		
	}


		
	while(1)
	{
			
		if (rtc_check(timerDac))
		{
			timerDac	= rtc_set(cfg.timerDac);
			
			uint16_t val = 123;			
			ad5660_setOutput(&ad5660_1, AD5660_PWR_NORMAL, val);
			ad5660_setOutput(&ad5660_2, AD5660_PWR_NORMAL, val);	
			
			sprintf(buffer, "{\"ad5660_1:\":1,\"ad5660_2:\":1 }\r\n", val,val);	
			usart_puts(&usartC0, buffer);
		}				
	
		if(rtc_check(timerGui))		// 500 mSec
		{
			timerGui	= rtc_set(cfg.timerGui);
			gui_update(&gui);							
		}			
			
	}	// end while
};


