#ifndef _MAIN_H
#define _MAIN_H

#undef F_CPU										// set for delays
#define F_CPU 32000000UL							// should always precede <avr/delay.h>


#include <stdlib.h>
#include <avr/io.h>
#include <stdio.h>
#include <stddef.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <string.h>

// lib
#include "macros.h"
#include "avr_compiler.h"
#include "clksys_driver.h"
#include "usart_driver.h"
#include "spi_driver.h"
#include "twi_master_driver.h"
#include "awex_driver.h"
#include "ac_driver.h"
#include "usart_driver.h"
#include "eeprom_driver.h"
#include "ssd1306_driver.h"
#include "wdt_driver.h"
#include "ad5660_driver.h"

// dev
#include "gui.h"
#include "gui_screens.h"
#include "rtc.h"
#include "base64_enc.h"



//#################################### Defines
#define	BAUDRATE_C0						115200						// 115200, 57600, 256000, ...
#define	BAUDRATE_D0						115200						// 115200, 57600, 256000, ...
#define	OLED_ADR						0x78


typedef struct cfg_data_t {

	uint16_t	timerDac;
	uint16_t	timerGui;
	
	// accu
	uint8_t		acpabl;					// accu pack state
	uint8_t		acpacg;
	uint8_t		acpapg;
	uint8_t		acpaup;					// accu pack state
	uint16_t	acpapr;					// accu pack percent 100.0%
	uint16_t	acpama;					// accu pack percent 100.0%
	uint16_t	acpamv;					// accu pack voltage in mv
	
	uint8_t		accxup;					// accu cel1 voltage in mv
	uint16_t	acc1mv;					// accu cel1 voltage in mv
	uint16_t	acc2mv;					// accu cel2 voltage in mv
	uint16_t	acc3mv;					// accu cel3 voltage in mv
	uint16_t	acc4mv;					// accu cel4 voltage in mv
	uint16_t	acc5mv;					// accu cel5 voltage in mv

	uint16_t	acc1bl;
	uint16_t	acc2bl;
	uint16_t	acc3bl;
	uint16_t	acc4bl;
	uint16_t	acc5bl;
	
	// trigger
	uint8_t		trSt;					// trigger state
	uint8_t		trup;
	uint8_t		trSend;
	uint8_t		trOfUs;					// trigger offset in uSec
	uint16_t	trVeUs;					// trigger ve
	uint16_t	trHeUs;					// trigger he
	uint16_t	trNeUs;					// trigger ne
	uint16_t	trLvMv;					// trigger dac trigger comperator level
	uint32_t	trms;					// trigger measuring time
	
	// wifi
	uint8_t		wiup;
	uint8_t		wiip[16];				// wifi "aaa.bbb.ccc.ddd"
	uint16_t	wist;					// wifi idle = 0, associating = 1, configuration = 2, ready = 3, failure = 4, disconnect = 5, SSIDNotFound = 6, undefined = 7
	uint16_t	wisig;					// wifi 0 - 100%

} cfg_data_t;



TWI_Master_t		twiMasterE;
SPI_Master_t		spiMasterC;
USART_data_t		usartC0;					// USART data struct used in example.
USART_data_t		usartD0;					// USART data struct used in example.


ad5660_data_t		ad5660_1;
ad5660_data_t		ad5660_2;

cfg_data_t			cfg;

ssd1306_data_t		ssd1306;
gui_data_t			gui;

#endif
